#include <pic.h>
#include <stdio.h>
#include<htc.h>
#include "clk_freq.h"
#include "always.h"
#include "delay.h"

#define _XTAL_FREQ 16000000

#define Drive1	RC0 
#define Drive2	RC1
#define Drive3	RC2
#define Drive4	RC3

float LDR1;
float LDR2;
float LDR3;
float LDR4;
//float diff=0.5;
//float diff=0.8;
//float diff=1.0;
//float diff=1.5;
//float diff=1.8;
//float diff=2.0;
//float diff=2.5;
//float diff=2.8;
float diff=2.0;

float Gain=1.0;
bit value_saved_flag=0;
unsigned char EEPROM_ADDRESS=0x00;

float Readadc( unsigned char ch);
void Value_compare_1(void);
void Value_compare_2(void);
void Wait(unsigned int delay);
void initADC(void);

void ewrite(unsigned char data, unsigned char address); //TO write failure on eeprom
unsigned char eread(unsigned char address);                //TO read failures from EEPROM
void eerase(unsigned char address);               //to Erase eeprom


void main()
{
	TRISA=0xFF;
	TRISC=0x10;
	
	Drive1=0; 
	Drive2=0;
	Drive3=0;
	Drive4=0;
	
	LDR1=0.0;
	LDR2=0.0;
	LDR3=0.0;
	LDR4=0.0;
	
	while(1)
	{
		LDR1=Readadc(1);
		LDR2=Readadc(2);
		LDR3=Readadc(3);
		LDR4=Readadc(4);

		if (!RC4)
		{
			RC5=1;	//indication that values has been stored eeprom
			value_saved_flag=1;
		}
		else
			RC5=0;
		if ((value_saved_flag ==1) && (RC4==1))
		{
			Value_compare_1();
			Value_compare_2();
			asm("CLRWDT");
		}
	}
}

float Readadc(unsigned char ch)
{
 	int ADC_DIG;
	float ADC_DEC;
	if(ch==1)
	{
		ADCON0=0b10000001;
	}
 	else if(ch==2)
	{
		ADCON0=0b10001001;
	}
	else if(ch==3)
	{
		ADCON0=0b10010001;
	}
	else if(ch==4)
	{
		ADCON0=0b10011001;
	}
	else
	{
		ADCON0=ADCON0;
	}

	__delay_us(20);
	GODONE = 1;
	while(GODONE);
	ADC_DIG=(ADRESH<<8) + ADRESL;
	ADC_DEC = (ADC_DIG*5.0* Gain)/1024;
	if (!RC4)
	{
		
		ewrite(ADRESH,  EEPROM_ADDRESS);
		EEPROM_ADDRESS+=1;
		ewrite(ADRESL,  EEPROM_ADDRESS);
		EEPROM_ADDRESS+=1;
	}	

	return ADC_DEC;
	asm("CLRWDT");
}

void Value_compare_1(void)
{

	if(LDR1>LDR2)
	{
  		
  		if((LDR1-LDR2) > diff)
  		{
   			Drive1 = 0;
   			Drive2 = 1;
  		}
  		
	}
	else if(LDR2>LDR1)
	{
		if((LDR2-LDR1) > diff)
  		{
   			Drive1 = 1;
   			Drive2 = 1;
  		}
  		
 	}	
	else
  		{ 
			Drive1 = 0;
    		Drive2 = 0; 
		}  		
 asm("CLRWDT");
}
void Value_compare_2(void)
{


	if(LDR3>LDR4)
	{
  		
  		if((LDR3-LDR4) > diff)
  		{
   			Drive3 = 0;
   			Drive4 = 1;
  		}
  		
	}
	else if(LDR4>LDR3)
	{
		if((LDR4-LDR3) > diff)
  		{
   			Drive3 = 1;
   			Drive4 = 1;
  		}
  		
 	}	
	else
  		{ 
			Drive3 = 0;
    		Drive4 = 0; 
		}  		
 asm("CLRWDT");
}

void Wait(unsigned int delay)
{
	for(;delay;delay--);
	__delay_ms(100);
}

void initADC(void)
{
	ADCON1=0b10000000;
	ADCON0=0b10000101;
	ADIF=0;
	ADIE=0;
	PEIE=0;
	GIE=0;
asm("CLRWDT");

}


unsigned char eread(unsigned char address)   //Read From EEPROM and if failures display on lcd
{ 
	unsigned char r = 0x00;
  	EEADR = address;
  	RD = 1;
 	wait(2000);
  	r = EEDATA;
	return r;
  
}
void ewrite(unsigned char data, unsigned char address)
{
	EEADR = address;
 	EEDATA = data;
 	WREN = 1;
 	EECON2 = 0x55; // protection sequence
 	EECON2 = 0xaa;
 	WR = 1;  // begin programming sequnce
 	//wait(2000);
 
}
/*void eerase(unsigned char address)
{ 
	
     	
     	EEADR = address;
     	ewrite(0x00);
  
}*/